package document
{
	import com.vulcan.application.frame.itemBase.WindowItem;
	import com.vulcan.application.framework.NameUtils;
	import com.vulcan.application.framework.behavior.SubSystem;
	import com.vulcan.application.framework.behavior.manager.FrameKeyManager;
	import com.vulcan.application.framework.model.graphic.GraphicsPX;
	import com.vulcan.application.framework.view.components.items.IGraphicItem;
	import com.vulcan.application.framework.view.mediators.DockFrameMED;
	import com.vulcan.application.framework.view.mediators.items.GraphicItemMED;
	import com.vulcan.application.framework.view.mediators.items.UIDocumentItemMEDBase;
	import com.vulcan.application.framework.view.mediators.layout.GraphicItemUpdateUtils;
	import com.vulcan.application.framework.view.mediators.layout.LayoutStyleUtils;
	import com.vulcan.application.generator.info.ItemInfo;
	import com.vulcan.application.uicontrol.SelectEvent;
	import com.vulcan.application.uicontrol.UiCanvas;
	import com.vulcan.application.uicontrol.dragers.display.DragerEvent;
	import com.vulcan.application.uicontrol.resizers.display.ResizerEvent;
	import com.vulcan.application.uicontrol.selection.SelectItem;
	
	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	import flash.utils.getQualifiedClassName;
	
	import mx.core.IVisualElementContainer;
	import mx.core.UIComponent;

	/**  
	 * 
	 * @Project : Vulcan Project
	 * @Author : Park Dong-il (pdi1066@naver.com)
	 * @Date : 2010. 5. 30.
	 */
	
	public class DocumentItemMED extends UIDocumentItemMEDBase
	{
		public function DocumentItemMED(mediatorName:String, windowItem:WindowItem)
		{
			super(mediatorName, windowItem);
		}
		
		public static const NAME:String = getQualifiedClassName(DocumentItemMED);
		public static const NAME_TYPE:String = NameUtils.TYPE_DOCUMENT;
		
		/////////////////////////////////
		//  Skin
		/////////////////////////////////
		
		override protected function get canvas():IVisualElementContainer
		{
			return DocumentItem(instance).uiCanvas;
		}
		
		override public function setSelected(key:IGraphicItem, selected:Boolean):void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			uiCanvas.setSelected(UIComponent(key), selected);
		}
		
		override public function setSelectable(key:IGraphicItem, selectable:Boolean):void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			uiCanvas.setSelectable(UIComponent(key), selectable);
		}
		
		/////////////////////////////////
		// 시작
		/////////////////////////////////
		
		/** @inheritDoc */
		override public function onRegister():void
		{
			super.onRegister();
			createKeyEvent();
		}
		
		// Mediator가 삭제될때
		override public function onRemove( ):void
		{
			super.onRemove();
			removeKeyEvent();
		}
		
		/////////////////////////////////
		// 키이벤트
		/////////////////////////////////
		
		protected function createKeyEvent():void
		{
			SubSystem(facade).managers.keyManager.add(FrameKeyManager.KEY_UP, keyUndo);
			SubSystem(facade).managers.keyManager.add(FrameKeyManager.KEY_UP, keyRedo);
			
			SubSystem(facade).managers.keyManager.add(FrameKeyManager.KEY_UP, deleteGraphicItem);
			
			SubSystem(facade).managers.keyManager.add(FrameKeyManager.KEY_UP, copyGraphicItem);
			SubSystem(facade).managers.keyManager.add(FrameKeyManager.KEY_UP, pasteGraphicItem);
		}
		
		protected function removeKeyEvent():void
		{
			SubSystem(facade).managers.keyManager.remove(FrameKeyManager.KEY_UP, keyUndo);
			SubSystem(facade).managers.keyManager.remove(FrameKeyManager.KEY_UP, keyRedo);
			
			SubSystem(facade).managers.keyManager.remove(FrameKeyManager.KEY_UP, deleteGraphicItem);
			
			SubSystem(facade).managers.keyManager.remove(FrameKeyManager.KEY_UP, copyGraphicItem);
			SubSystem(facade).managers.keyManager.remove(FrameKeyManager.KEY_UP, pasteGraphicItem);
		}
		
		// ctrl + z
		private function keyUndo(focusItem:WindowItem, event:KeyboardEvent):void
		{
			if(windowItem == focusItem)
			{
				// Keyboard.Z
				if ( !event.altKey && !event.shiftKey && event.ctrlKey && event.keyCode == 90 ){
					SubSystem(facade).commands.undo();
				}
			}
		}
		
		// ctrl + y
		private function keyRedo(focusItem:WindowItem, event:KeyboardEvent):void
		{
			if(windowItem == focusItem)
			{
				// Keyboard.Y
				if ( !event.altKey && !event.shiftKey && event.ctrlKey && event.keyCode == 89 ){
					SubSystem(facade).commands.redo();
				}
			}
		}
		
		// delete 키
		private function deleteGraphicItem(focusItem:WindowItem, event:KeyboardEvent):void
		{
			if(windowItem == focusItem)
			{
				if ( !event.altKey && !event.shiftKey && !event.ctrlKey && event.keyCode == Keyboard.DELETE ){
					SubSystem(facade).commands.removeSelectAll();
				}
			}
		}
		
		// ctrl + c
		private function copyGraphicItem(focusItem:WindowItem, event:KeyboardEvent):void
		{
			if(windowItem == focusItem)
			{
				// Keyboard.C
				if ( !event.altKey && !event.shiftKey && event.ctrlKey && event.keyCode == 67 ){
					SubSystem(facade).commands.copyGraphicItem();
				}
			}
		}
		
		// ctrl + v
		private function pasteGraphicItem(focusItem:WindowItem, event:KeyboardEvent):void
		{
			if(windowItem == focusItem)
			{
				// Keyboard.V
				if ( !event.altKey && !event.shiftKey && event.ctrlKey && event.keyCode == 86 ){
					SubSystem(facade).commands.pasteGraphicItem();
				}
				
			}
		}
		
		/////////////////////////////////
		// 이벤트
		/////////////////////////////////
		
		override protected function createEvent():void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			
			//Select
			uiCanvas.addEventListener(SelectEvent.SELECT_DATA_UPDATE, onSelectDataUpdate);
			uiCanvas.addEventListener(SelectEvent.SELECT_DATA_RESET, onSelectDataReset);
			
			// Drag
			uiCanvas.addEventListener(DragerEvent.DRAG_START, onDragStart);
			//uiCanvas.addEventListener(DragerEvent.DRAG_MOVE, onDragMove);
			uiCanvas.addEventListener(DragerEvent.DRAG_END, onDragEnd);
			
			// Resize
			uiCanvas.addEventListener(ResizerEvent.RESIZE_START, onResizeStart);
			//uiCanvas.addEventListener(ResizerEvent.RESIZE_MOVE, onResizeMove);
			uiCanvas.addEventListener(ResizerEvent.RESIZE_END, onResizeEnd);
			
		}
		
		override protected function removeEvent():void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			
			//Select
			uiCanvas.removeEventListener(SelectEvent.SELECT_DATA_UPDATE, onSelectDataUpdate);
			uiCanvas.removeEventListener(SelectEvent.SELECT_DATA_RESET, onSelectDataReset);
			
			// Drag
			uiCanvas.removeEventListener(DragerEvent.DRAG_START, onDragStart);
			//uiCanvas.removeEventListener(DragerEvent.DRAG_MOVE, onDragMove);
			uiCanvas.removeEventListener(DragerEvent.DRAG_END, onDragEnd);
			
			// Resize
			uiCanvas.removeEventListener(ResizerEvent.RESIZE_START, onResizeStart);
			//uiCanvas.removeEventListener(ResizerEvent.RESIZE_MOVE, onResizeMove);
			uiCanvas.removeEventListener(ResizerEvent.RESIZE_END, onResizeEnd);
			
		}
		
		//----------------------------------
		// Selection
		//----------------------------------
		
		private function checkSelectDocument():void
		{
			if(documentItemInfo.isSelected){
				return;
			}
			
			// Document가 선택상태가 아니면 GraphicItem을 선택할 수 없다.
			// Document 선택 먼저한다.
			var name:String = NameUtils.getName(DockFrameMED);
			var frameSystemMED:DockFrameMED = facade.retrieveMediator(name) as DockFrameMED;
			frameSystemMED.selectDocument(documentUID);
		}
		
		// 선택 유무와는 상관없이 내용이 변경된 객체에 대해 발생한다.
		private function onSelectDataUpdate(event:SelectEvent):void
		{
			checkSelectDocument();
			
			var list:Array = event.items;
			const len:int = list.length;
			if(len < 1) return;
			
			for each(var selectItem:SelectItem in list)
			{
				invalidateSelection(selectItem);
			}
		}
		
		private function onSelectDataReset(event:SelectEvent):void
		{
			trace("<<onSelectDataReset>>");
		}
		
		private function invalidateSelection(selectItem:SelectItem):void
		{
			var graphicsPX:GraphicsPX = SubSystem(facade).getProxy(GraphicsPX, documentUID) as GraphicsPX;
			
			var graphic:IGraphicItem = selectItem.key as IGraphicItem;
			var info:ItemInfo = graphicsPX.getItemInfo( graphic.instance.uid );
			
			if(info == null) return;
			if(info.isSelected == selectItem.selected && info.isSelectable == selectItem.selectable){
				return;
			}
			
			var newInfo:ItemInfo = info.clone();
			var med:GraphicItemMED = getGraphicItemMED(info.uid);
			med.updateSelectInfo(newInfo, selectItem);
			
			SubSystem(facade).commands.select(newInfo);
		}
		
		private function getGraphicItemMED(graphicUID:String):GraphicItemMED
		{
			var name:String = NameUtils.getName(GraphicItemMED, graphicUID);
			var mediator:GraphicItemMED = facade.retrieveMediator(name) as GraphicItemMED;
			return mediator;
		}
		
		//----------------------------------
		// Resize, Drag
		//----------------------------------
		
		private function onDragStart(event:DragerEvent):void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			var graphicItem:IGraphicItem = uiCanvas.dragUtil.target as IGraphicItem;
			
			_changePropertiesList = {
					x : GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "x"),
					y : GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "y")
			};
		}
		
		private function onDragEnd(event:DragerEvent):void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			var graphicItem:IGraphicItem = uiCanvas.resizeUtil.target as IGraphicItem;
			
			var x:* = GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "x");
			var y:* = GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "y");
			
			// 변경된 목록
			var properties:Object = {};
			
			if(_changePropertiesList.x != x){
				properties.x = x;
			}
			if(_changePropertiesList.y != y){
				properties.y = y;
			}
			
			_changePropertiesList = null;
			
			checkSelectDocument();
			invalidateDisplay(properties);
		}
		
		private var _changePropertiesList:Object;
		private function onResizeStart(event:ResizerEvent):void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			var graphicItem:IGraphicItem = uiCanvas.resizeUtil.target as IGraphicItem;
			
			_changePropertiesList = {
					x : GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "x"),
					y : GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "y"),
					width : GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "width"),
					height : GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "height")
			};
		}
		
		private function onResizeEnd(event:ResizerEvent):void
		{
			var uiCanvas:UiCanvas = canvas as UiCanvas;
			var graphicItem:IGraphicItem = uiCanvas.resizeUtil.target as IGraphicItem;
			
			var x:* = GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "x");
			var y:* = GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "y");
			var width:* = GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "width");
			var height:* = GraphicItemUpdateUtils.getInstanceProperty(graphicItem, "height");
			
			// 변경된 목록
			var properties:Object = {};
			
			if(_changePropertiesList.x != x){
				properties.x = x;
			}
			if(_changePropertiesList.y != y){
				properties.y = y;
			}
			if(_changePropertiesList.width != width){
				properties.width = width;
			}
			if(_changePropertiesList.height != height){
				properties.height = height;
			}
			
			_changePropertiesList = null;
			
			checkSelectDocument();
			invalidateDisplay(properties);
		}
		
		private function invalidateDisplay(properties:Object):void
		{
			// 선택상태의 아이템
			var graphicsPX:GraphicsPX = SubSystem(facade).getProxy(GraphicsPX, documentUID) as GraphicsPX;
			var infos:Array = graphicsPX.getSelectedItemInfos();
			
			if(infos.length < 1){
				return;
			}
			
			if(infos.length > 1){
				throw new Error("다중 선택은 아직 구현되지 않았습니다.");
			}
			
			var info:ItemInfo = infos[0];
			var newInfo:ItemInfo = info.clone();
			
			// 변경값 전달
			var med:GraphicItemMED = getGraphicItemMED(newInfo.uid);
			//med.updateIteminfo(newInfo);
			
			var changeNameAr:Array = [];
			for(var propName:String in properties)
			{
				changeNameAr.push(propName);
				newInfo.item.propertyVO[propName] = properties[propName];
			}
			
			// Layout과 스타일 충돌을 조율
			LayoutStyleUtils.checkLayout(med.graphicItem, newInfo, LayoutStyleUtils.CHANGE_PROPERTY, changeNameAr);
			
			SubSystem(facade).commands.modify(newInfo);
		}
		
		////////////////////////////////////
		// End
		////////////////////////////////////
	}
}