package items.component
{
	/**  
	 * 
	 * @Project : Vulcan Project
	 * @Author : Park Dong-il (pdi1066@naver.com)
	 * @Date : 2010. 7. 12.
	 */
	import com.vulcan.application.frame.itemBase.DefineWindowItem;
	import com.vulcan.application.frame.itemBase.WindowItem;
	import com.vulcan.application.framework.NameUtils;
	import com.vulcan.application.framework.NoteUtils;
	import com.vulcan.application.framework.behavior.SubSystem;
	import com.vulcan.application.framework.behavior.manager.ProcessManager;
	import com.vulcan.application.framework.component.catalog.component.ComponentListEvent;
	import com.vulcan.application.framework.component.catalog.swc.SwcComponentList;
	import com.vulcan.application.framework.model.DocumentsPX;
	import com.vulcan.application.framework.model.data.swc.DocumentDATA;
	import com.vulcan.application.framework.model.data.swc.DocumentVO;
	import com.vulcan.application.framework.model.document.CodePX;
	import com.vulcan.application.framework.view.mediators.items.WindowItemMEDBase;
	import com.vulcan.application.generator.code.CatalogUtils;
	import com.vulcan.application.generator.info.ItemInfo;
	import com.vulcan.application.swcparser.SWC;
	import com.vulcan.application.swcparser.SWCReader;
	import com.vulcan.core.type.map.HashMap;
	import com.vulcan.mvc.interfaces.INotification;
	import com.vulcan.mvc.patterns.mediator.Mediator;
	
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.MouseEvent;
	import flash.events.SecurityErrorEvent;
	import flash.system.ApplicationDomain;
	import flash.utils.getQualifiedClassName;
	
	import mx.core.ClassFactory;
	import mx.utils.NameUtil;
	
	import spark.components.Button;
	import spark.components.supportClasses.ItemRenderer;
	
	public class ComponentWindowItemMED extends WindowItemMEDBase
	{
		public function ComponentWindowItemMED(mediatorName:String, windowItem:WindowItem)
		{
			super(mediatorName, windowItem);
		}
		
		public static const NAME:String = getQualifiedClassName(ComponentWindowItemMED);
		public static const NAME_TYPE:String = NameUtils.TYPE_APPLICATION;
		
		/////////////////////////////////
		//  Skin
		/////////////////////////////////
		
		private function get dataList():SwcComponentList
		{
			return ComponentWindowItem(instance).dataList as SwcComponentList;
		}
		
		private function get addBtn():Button
		{
			return ComponentWindowItem(instance).addBtn as Button;
		}
		
		/////////////////////////////////
		// 시작
		/////////////////////////////////
		
		override public function onRegister():void
		{
			super.onRegister();
			addEvent();
			
			// 초기 세팅
			onDocumentSelected();
			
			// 드래그 아이템 추가 이벤트
			
		}
		
		// Mediator가 삭제될때
		override public function onRemove( ):void
		{
			super.onRemove();
			removeEvent();
		}
		
		private function addEvent():void
		{
			//더블클릭 이벤트 등록
			dataList.addEventListener(ComponentListEvent.ITEM_DOUBLE_CLICK, onDoubleClick);
			/*
			dataList.addEventListener(Event.COMPLETE, onSwcLoaded);
			dataList.addEventListener( IOErrorEvent.IO_ERROR, onSwcLoadError );
			dataList.addEventListener( SecurityErrorEvent.SECURITY_ERROR, onSwcLoadError );
			*/
			addBtn.addEventListener(MouseEvent.CLICK, onAddClick);
		}
		
		private function removeEvent():void
		{
			dataList.removeEventListener(ComponentListEvent.ITEM_DOUBLE_CLICK, onDoubleClick);
			/*
			dataList.removeEventListener(Event.COMPLETE, onSwcLoaded);
			dataList.removeEventListener( IOErrorEvent.IO_ERROR, onSwcLoadError );
			dataList.removeEventListener( SecurityErrorEvent.SECURITY_ERROR, onSwcLoadError );
			*/
			addBtn.removeEventListener(MouseEvent.CLICK, onAddClick);
		}
		
		private function onAddClick(event:Event):void
		{
			var data:XML = dataList.selectedItem as XML;
			if(!data){
				return;
			}
			
			var info:ItemInfo = new ItemInfo(data);
			
			info.isChild = false;
			info.addPointerUID = getSelectUID();
			SubSystem(facade).commands.add(info);
		}
		
		// 아이템 ADD
		private function onDoubleClick(event:ComponentListEvent):void
		{
			var data:XML = ItemRenderer(event.renderer).data as XML;
			var info:ItemInfo = new ItemInfo(data);
			
			info.isChild = false;
			trace("\n\n\n\n\n\n", this, "************************************isChild 값에 항상 주의하세요\n\n\n\n\n\n")
			info.addPointerUID = getSelectUID();
			SubSystem(facade).commands.add(info);
		}
		
		// 현재 code내에서 삽입위치를 나타내는 포인터 UID를 구한다.
		private function getSelectUID():String
		{
			var codePX:CodePX = SubSystem(facade).getProxy( CodePX ) as CodePX;
			return codePX.selectUID;
		}
		
		/////////////////////////////////
		// 선택 문서가 바뀌었을 때
		/////////////////////////////////
		
		private var currentDocumentKey:String;
		
		// 선택 문서가 바뀌었을 때
		override protected function onDocumentSelected():void
		{
			var info:ItemInfo = SubSystem(facade).internalAPI.getDocumentInfo();
			
			if(info == null){
				currentDocumentKey = null;
				return;
			}else{
				var key:String = info.documentType;
				currentDocumentKey = key;
			}
			/*
			updateCatalog();
			updateLibrary();
			*/
		}
		
		// 화면 초기화 로직 구현 - override
		override protected function initDisplay():void
		{
			dataList.filteredClassMap = null;
		}
		
		// 화면 갱신 로직 구현 - override
		override protected function updateDisplay():void
		{
			if(currentDocumentKey)
			{
				updateCatalog();
				updateLibrary();
				
			}else{
				deactivate();
			}
		}
		
		// API Catalog 구하기
		private function updateCatalog():void
		{
			// source : API catalog.xml
			var xml:XML = SubSystem(facade).librarys.getCatalog(currentDocumentKey);
			dataList.source = xml;
		}
		
		// SWC 라이브러리 구하기 - 문서 종류에 따라 지원되는 컴포넌트가 다르도록 Catalog 로부터 필터링한다.
		private function updateLibrary():void
		{
			// 초기화
			dataList.filteredClassMap = null;
			
			// 등록된 데이터를 활용
			var readers:Array = SubSystem(facade).librarys.getSwcReader(currentDocumentKey);
			for each(var reader:SWCReader in readers)
			{
				dataList.addSwcFilter(reader.swc);
			}
			
			var documentDATA:DocumentDATA = SubSystem(facade).getProxy(DocumentDATA) as DocumentDATA;
			var vo:DocumentVO = documentDATA.getDocumentVO(currentDocumentKey) as DocumentVO;
			var rootClassName:String = vo.rootClassName;
			
			// Main Application 필터링
			var classMap:HashMap = SubSystem(facade).librarys.getClassFilter(currentDocumentKey).clone();
			var keys:Array = classMap.getKeys();
			for each(var key:String in keys)
			{
				if(key == rootClassName)
				{
					trace(this, "에서 다음 클래스는 필터링 되었습니다. : ", key);
					classMap.remove(key);
				}
			}
			
			dataList.filteredClassMap = classMap;
		}
		
		/*
		private function filterFunction(item:Object):Boolean
		{
			if(!_filteredClassMap || _filteredClassMap.size() == 0){
				return false;
			}
			
			var className:String = item.@className;
			var visible:Boolean = _filteredClassMap.containsKey(className);
			
			return visible;
		}
		*/
		
		////////////////////////////////////
		// End
		////////////////////////////////////
	}
}



// 초기 세팅시 로드하지 않고 문서가 선택될때마다 swc를 동적으로 로드할 경우


/*
// SWC 라이브러리 구하기 - 문서 종류에 따라 지원되는 컴포넌트가 다르도록 Catalog 로부터 필터링한다.
private function updateLibrary():void
{
	dataList.filteredClassMap = null;
	
	var registed:Boolean = checkFilterMap();
	if(registed){
		trace("// 등록된 데이터를 활용");
		var key:String = currentDocumentType.key;
		dataList.filteredClassMap = _filterMapList.getValue(key);
		
	}else{
		trace("// 새로 로드");
		ProcessManager.protectedScreenON();
		loadLibrary();
	}
}

//------------------------------------
// 라이브러리 필터링 관리
//------------------------------------

// 라이브러리 필터링에 필요한 Map
private var _filterMapList:HashMap = new HashMap(HashMap);

// 문서 종류에 해당하는 filteredClassMap이 있는지 조사.
private function checkFilterMap():Boolean
{
	var key:String = currentDocumentType.key;
	return _filterMapList.containsKey(key);
}

//------------------------------------
// 로드된 swc 자원 관리
//------------------------------------

// 로드된 swc 자원 관리를 위한 Map
private var swcReaderMap:HashMap;



// 라이브러리 자원 삭제
private function removeSwcReaderGroup(key:String):void
{
	var readers:Array = swcReaderMap.getValue(key) as Array;
	for each(var reader:SWCReader in readers){
		reader.unload();
	}
}

////////////////////////////////////////
// 새로운 SWC 라이브러리 구하기 - 로드
////////////////////////////////////////

private var _totalLoadCount:int = 0;
private var _loadCount:int = 0;

private function loadLibrary():void
{
	var swcList:Array = currentDocumentType.swcUrlList;
	
	_totalLoadCount = swcList.length;
	_loadCount = 0;
	
	var readerAr:Array = [];
	for each(var url:String in swcList)
	{
		//trace("swc url : ", url);
		var reader:SWCReader = dataList.addSWC(url);
		readerAr.push(reader);
	}
	
	// 라이브러리 자원 추가
	if(swcReaderMap == null){
		swcReaderMap = new HashMap(Array);
	}
	swcReaderMap.put(currentDocumentType.key, readerAr);
}

// swc가 로드완료 될때마다 발생
private function onSwcLoaded(event:Event):void
{
	_loadCount++;
	getLibraryFilter();
}

private function onSwcLoadError(event:Event):void
{
	trace("\n\nERROR : 해당 문서의 SWC 라이브러리 로드에 실패했습니다.\\n");
	_loadCount++;
	getLibraryFilter();
}

private function getLibraryFilter():void
{
	//trace("onSwcLoaded", _loadCount, "/", _totalLoadCount)
	if(_totalLoadCount == _loadCount)
	{
		// 복제하여 등록
		var key:String = currentDocumentType.key;
		var value:HashMap = dataList.filteredClassMap.clone();
		
		// 라이브러리 자원 추가
		_filterMapList.put(key, value);
		
		ProcessManager.protectedScreenOFF();
	}
}

/////////////////////////////////
// 문서가 닫힐 때
/////////////////////////////////

// 항상 청취하는 Note
override protected function getStaticNotification():Array
{
	var noteList:Array = [
		NoteUtils.getApplicationNoteName(DocumentsPX.REMOVE_AFTER)
	];
	return noteList;
}

override public function handleNotification(note:INotification):void
{
	super.handleNotification(note);
	
	var noteName:String = note.getName();
	switch ( noteName )
	{
		case NoteUtils.getApplicationNoteName(DocumentsPX.REMOVE_AFTER):
			onRemoveAfter();
			break;
	}
}

//------------------------------------
// SWC 라이브러리 제거하기 - 같은 종류의 문서가 모두 닫혔을때
//------------------------------------

private function onRemoveAfter():void
{
	var documentsPX:DocumentsPX = SubSystem(facade).getProxy(DocumentsPX) as DocumentsPX;
	
	var key:String = currentDocumentType.key;
	var canRemove:Boolean = true;
	
	//같은 종류의 문서체크
	var ar:Array = documentsPX.getItemInfos();
	for each(var info:ItemInfo in ar)
	{
		var vo:DocumentVO = info.documentType as DocumentVO;
		if(vo){
			if(vo.key == key){
				canRemove = false;
				break;
			}
		}
	}
	
	if(canRemove == false){
		return;
	}
	
	trace("onRemoveAfter : ", key);
	
	// 라이브러리 자원 삭제
	var readers:Array = swcReaderMap.getValue(key) as Array;
	for each(var reader:SWCReader in readers){
		reader.unload();
	}
	
	// 관리 리스트에서 제거
	swcReaderMap.remove(key);
	_filterMapList.remove(key);
	
	// 필터링 업데이트
	updateLibrary();
}
*/







