package menubar
{
	import com.vulcan.application.framework.NameUtils;
	import com.vulcan.application.framework.NoteUtils;
	import com.vulcan.application.framework.behavior.CMDConstant;
	import com.vulcan.application.framework.behavior.SubSystem;
	import com.vulcan.application.framework.model.DocumentsPX;
	import com.vulcan.application.generator.info.ItemInfo;
	import com.vulcan.mvc.patterns.mediator.Mediator;
	
	import flash.utils.getQualifiedClassName;
	
	import mx.controls.MenuBar;
	import mx.events.MenuEvent;

	/**  
	 * 
	 * @Project : Vulcan Project
	 * @Author : Park Dong-il (pdi1066@naver.com)
	 * @Date : 2010. 5. 27.
	 */
	
	public class MenuToolBarMED extends Mediator
	{
		public function MenuToolBarMED(mediatorName:String=null, viewComponent:Object=null)
		{
			super(mediatorName, viewComponent);
		}
		
		public static const NAME:String = getQualifiedClassName(MenuToolBarMED);
		public static const NAME_TYPE:String = NameUtils.TYPE_APPLICATION;
		
		
		public function get instance():MenuBar
		{
			return viewComponent as MenuBar
		}
		
		/////////////////////////////////
		// Event Binding
		/////////////////////////////////
		
		/** @inheritDoc */
		override public function onRegister():void
		{
			instance.addEventListener(MenuEvent.ITEM_CLICK, onItemClick);
			
			var menuPX:MenuToolBarDATA = SubSystem(facade).getProxy(MenuToolBarDATA) as MenuToolBarDATA;
			var menu:XML = menuPX.getData() as XML;
			
			instance.dataProvider = menu.children();
		}
		
		// Mediator가 삭제될때
		override public function onRemove( ):void
		{
			instance.removeEventListener(MenuEvent.ITEM_CLICK, onItemClick);
			instance.dataProvider = null;
		}
		
		protected function onItemClick(event:MenuEvent):void
		{
			var commendConst:String = event.item.@data;
			if(!commendConst) return;
			
			callCommend(commendConst);
		}
		
		/////////////////////////////////
		// Command 호출 API 정의
		/////////////////////////////////
		
		// 파일메뉴의 항목 선택 등으로 호출할 수 있는 인터페이스
		protected function callCommend(commendString:String):void
		{
			var str:String = CMDConstant[commendString];
			if(!str) return;
			
			var info:ItemInfo;
			var name:String;
			var documentsPX:DocumentsPX;
			
			switch(str)
			{
				case CMDConstant.NEW_DOCUMENT:
				{
					SubSystem(facade).commands.newDocument();
					break;
				}
					
				case CMDConstant.CLOSE_DOCUMENT:
				{
					info = getCurrentDocumentIteminfo();
					SubSystem(facade).commands.closeDocument(info);
					break;
				}
					
				case CMDConstant.UNDO:
				{
					if(!NoteUtils.CURRENT_DOCUMENT_UID) return;
					SubSystem(facade).commands.undo();
					break;
				}
					
				case CMDConstant.REDO:
				{
					if(!NoteUtils.CURRENT_DOCUMENT_UID) return;
					SubSystem(facade).commands.redo();
					break;
				}
					
				case CMDConstant.SAVE_DOCUMENT:
				{
					info = getCurrentDocumentIteminfo();
					SubSystem(facade).commands.saveDocument(info);
					break;
				}
					
				case CMDConstant.SAVE_AS_DOCUMENT:
				{
					info = getCurrentDocumentIteminfo();
					SubSystem(facade).commands.saveAsDocument(info);
					break;
				}
					
				case CMDConstant.OPEN_DOCUMENT:
				{
					SubSystem(facade).commands.openDocument();
					break;
				}
					
				case CMDConstant.OPEN_WINDOW_ITEM:
				{
					SubSystem(facade).commands.openWindowItem("property");
					break;
				}
					
				case CMDConstant.CLOSE_WINDOW_ITEM:
				{
					SubSystem(facade).commands.closeWindowItem("property");
					break;
				}
					
				case CMDConstant.COPY_GRAPHIC_ITEM:
				{
					SubSystem(facade).commands.copyGraphicItem();
					break;
				}					
					
				case CMDConstant.PASTE_GRAPHIC_ITEM:
				{
					SubSystem(facade).commands.pasteGraphicItem();
					break;
				}
			}
			
		}
		
		private function getCurrentDocumentIteminfo():ItemInfo
		{
			// DocumentsPX
			var documentsPX:DocumentsPX = SubSystem(facade).getProxy(DocumentsPX) as DocumentsPX;
			var info:ItemInfo = documentsPX.getItemInfo(NoteUtils.CURRENT_DOCUMENT_UID);
			return info;
		}
		
		
		////////////////////////////////////
		// End
		////////////////////////////////////
	}
}